@@ -1,82 +1,72 @@
-#[derive(structopt::StructOpt)]
-#[structopt(
-        setting = structopt::clap::AppSettings::UnifiedHelpMessage,
-        setting = structopt::clap::AppSettings::DeriveDisplayOrder,
-        setting = structopt::clap::AppSettings::DontCollapseArgsInUsage,
-        setting = concolor_clap::color_choice(),
+#[derive(clap::Parser)]
+#[clap(about, author, version)]
+#[clap(
+        setting = clap::AppSettings::DeriveDisplayOrder,
+        dont_collapse_args_in_usage = true,
+        color = concolor_clap::color_choice(),
     )]
-#[structopt(group = structopt::clap::ArgGroup::with_name("mode").multiple(false))]
+#[clap(group = clap::ArgGroup::new("mode").multiple(false))]
 pub struct Args {
     /// Rebase the selected stacks
-    #[structopt(short, long, group = "mode")]
+    #[clap(short, long, group = "mode")]
     pub rebase: bool,
 
     /// Pull the parent branch and rebase onto it.
-    #[structopt(long)]
+    #[clap(long)]
     pub pull: bool,
 
     /// Push all ready branches
-    #[structopt(long)]
+    #[clap(long)]
     pub push: bool,
 
     /// Which branch stacks to include
-    #[structopt(
-        short,
-        long,
-        possible_values(&git_stack::config::Stack::variants()),
-        case_insensitive(true),
-    )]
+    #[clap(short, long, arg_enum)]
     pub stack: Option<git_stack::config::Stack>,
 
     /// Branch to evaluate from (default: most-recent protected branch)
-    #[structopt(long)]
+    #[clap(long)]
     pub base: Option<String>,
 
     /// Branch to rebase onto (default: base)
-    #[structopt(long)]
+    #[clap(long)]
     pub onto: Option<String>,
 
     /// Action to perform with fixup-commits
-    #[structopt(
-        long,
-        possible_values(&git_stack::config::Fixup::variants()),
-        case_insensitive(true),
-    )]
+    #[clap(long, arg_enum)]
     pub fixup: Option<git_stack::config::Fixup>,
 
     /// Repair diverging branches.
-    #[structopt(long, overrides_with("no-repair"))]
+    #[clap(long, overrides_with("no-repair"))]
     repair: bool,
-    #[structopt(long, overrides_with("repair"), hidden(true))]
+    #[clap(long, overrides_with("repair"), hide = true)]
     no_repair: bool,
 
-    #[structopt(short = "n", long)]
+    #[clap(short = 'n', long)]
     pub dry_run: bool,
 
-    #[structopt(
-        long,
-        possible_values(&git_stack::config::Format::variants()),
-        case_insensitive(true),
-    )]
+    #[clap(long, arg_enum)]
     pub format: Option<git_stack::config::Format>,
 
+    #[clap(long, arg_enum)]
+    pub show_commits: Option<git_stack::config::ShowCommits>,
+
     /// See what branches are protected
-    #[structopt(long, group = "mode")]
+    #[clap(long, group = "mode")]
     pub protected: bool,
 
     /// Append a protected branch to the repository's config (gitignore syntax)
-    #[structopt(long, group = "mode")]
+    #[clap(long, group = "mode")]
     pub protect: Option<String>,
 
     /// Write the current configuration to file with `-` for stdout
-    #[structopt(long, group = "mode")]
+    #[clap(long, parse(from_os_str), group = "mode")]
     pub dump_config: Option<std::path::PathBuf>,
 
-    #[structopt(flatten)]
+    #[clap(flatten)]
     pub(crate) color: concolor_clap::Color,
 
-    #[structopt(flatten)]
-    pub verbose: clap_verbosity_flag::Verbosity,
+    #[clap(flatten)]
+    pub verbose: clap_verbosity_flag::Verbosity<clap_verbosity_flag::InfoLevel>,
 }
 
 impl Args {
@@ -89,6 +79,7 @@ impl Args {
             push_remote: None,
             pull_remote: None,
             show_format: self.format,
+            show_commits: self.show_commits,
             show_stacked: None,
             auto_fixup: None,
             auto_repair: None,
@@ -107,6 +98,17 @@ fn resolve_bool_arg(yes: bool, no: bool) -> Option<bool> {
         (true, false) => Some(true),
         (false, true) => Some(false),
         (false, false) => None,
-        (_, _) => unreachable!("StructOpt should make this impossible"),
+        (_, _) => unreachable!("clap should make this impossible"),
+    }
+}
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn verify_app() {
+        use clap::CommandFactory;
+        Args::command().debug_assert()
     }
 }
