@@ -2,9 +2,7 @@
 #![allow(clippy::let_and_return)]
 #![allow(clippy::if_same_then_else)]
 
-use std::io::Write;
-
-use structopt::StructOpt;
+use clap::Parser;
 
 mod args;
 mod config;
@@ -18,20 +16,21 @@ fn main() {
 
 fn run() -> proc_exit::ExitResult {
     // clap's `get_matches` uses Failure rather than Usage, so bypass it for `get_matches_safe`.
-    let args = match args::Args::from_args_safe() {
+    let args = match args::Args::try_parse() {
         Ok(args) => args,
         Err(e) if e.use_stderr() => {
-            return Err(proc_exit::Code::USAGE_ERR.with_message(e));
+            let _ = e.print();
+            return proc_exit::Code::USAGE_ERR.ok();
         }
         Err(e) => {
-            writeln!(std::io::stdout(), "{}", e)?;
+            let _ = e.print();
             return proc_exit::Code::SUCCESS.ok();
         }
     };
 
     args.color.apply();
-    let colored_stdout = concolor_control::get(concolor_control::Stream::Stdout).ansi_color();
-    let colored_stderr = concolor_control::get(concolor_control::Stream::Stderr).ansi_color();
+    let colored_stdout = concolor::get(concolor::Stream::Stdout).ansi_color();
+    let colored_stderr = concolor::get(concolor::Stream::Stderr).ansi_color();
 
     git_stack::log::init_logging(args.verbose.clone(), colored_stderr);
 
