@@ -25,6 +25,7 @@ struct State {
     protect_commit_time: std::time::SystemTime,
 
     show_format: git_stack::config::Format,
+    show_commits: git_stack::config::ShowCommits,
     show_stacked: bool,
 }
 
@@ -85,6 +86,7 @@ impl State {
         let protect_commit_age = repo_config.protect_commit_age();
         let protect_commit_time = std::time::SystemTime::now() - protect_commit_age;
         let show_format = repo_config.show_format();
+        let show_commits = repo_config.show_commits();
         let show_stacked = repo_config.show_stacked();
 
         repo.set_push_remote(repo_config.push_remote());
@@ -107,6 +109,13 @@ impl State {
             .with_code(proc_exit::Code::USAGE_ERR)?;
 
         let stacks = match (base, onto, repo_config.stack()) {
+            (Some(base), Some(onto), git_stack::config::Stack::All) => {
+                vec![StackState {
+                    base,
+                    onto,
+                    branches: branches.all(),
+                }]
+            }
             (Some(base), None, git_stack::config::Stack::All) => {
                 let onto = base.clone();
                 vec![StackState {
@@ -203,6 +212,7 @@ impl State {
             protect_commit_time,
 
             show_format,
+            show_commits,
             show_stacked,
         })
     }
@@ -420,9 +430,13 @@ fn plan_changes(state: &State, stack: &StackState) -> eyre::Result<git_stack::gi
     if let Some(protect_commit_count) = state.protect_commit_count {
         git_stack::graph::protect_large_branches(&mut graph, protect_commit_count);
     }
-    git_stack::graph::protect_old_branches(&mut graph, state.protect_commit_time);
+    git_stack::graph::protect_old_branches(
+        &mut graph,
+        state.protect_commit_time,
+        &[state.head_commit.id],
+    );
     if let Some(user) = state.repo.user() {
-        git_stack::graph::protect_foreign_branches(&mut graph, &user);
+        git_stack::graph::protect_foreign_branches(&mut graph, &user, &[state.head_commit.id]);
     }
 
     let mut dropped_branches = Vec::new();
@@ -486,9 +500,13 @@ fn push(state: &mut State) -> eyre::Result<()> {
     if let Some(protect_commit_count) = state.protect_commit_count {
         git_stack::graph::protect_large_branches(&mut graph, protect_commit_count);
     }
-    git_stack::graph::protect_old_branches(&mut graph, state.protect_commit_time);
+    git_stack::graph::protect_old_branches(
+        &mut graph,
+        state.protect_commit_time,
+        &[state.head_commit.id],
+    );
     if let Some(user) = state.repo.user() {
-        git_stack::graph::protect_foreign_branches(&mut graph, &user);
+        git_stack::graph::protect_foreign_branches(&mut graph, &user, &[state.head_commit.id]);
     }
 
     git_stack::graph::pushable(&mut graph);
@@ -508,10 +526,17 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
     let mut old_stacks = Vec::new();
     let mut foreign_stacks = Vec::new();
 
+    let abbrev_graph = match state.show_format {
+        git_stack::config::Format::Silent => false,
+        git_stack::config::Format::List => false,
+        git_stack::config::Format::Graph => true,
+        git_stack::config::Format::Debug => true,
+    };
+
     let mut graphs = Vec::with_capacity(state.stacks.len());
     for stack in state.stacks.iter() {
         let graphed_branches = stack.graphed_branches();
-        if graphed_branches.len() == 1 {
+        if graphed_branches.len() == 1 && abbrev_graph {
             let branches = graphed_branches.iter().next().unwrap().1;
             if branches.len() == 1 && branches[0].id != state.head_commit.id {
                 empty_stacks.push(format!("{}", palette_stderr.info.paint(&branches[0].name)));
@@ -551,21 +576,27 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
                 );
             }
         }
-        old_stacks.extend(
-            git_stack::graph::trim_old_branches(
-                &mut graph,
-                state.protect_commit_time,
-                &[state.head_commit.id],
-            )
-            .into_iter()
-            .map(|b| format!("{}", palette_stderr.warn.paint(b))),
-        );
-        if let Some(user) = state.repo.user() {
-            foreign_stacks.extend(
-                git_stack::graph::trim_foreign_branches(&mut graph, &user, &[state.head_commit.id])
+        if abbrev_graph {
+            old_stacks.extend(
+                git_stack::graph::trim_old_branches(
+                    &mut graph,
+                    state.protect_commit_time,
+                    &[state.head_commit.id],
+                )
+                .into_iter()
+                .map(|b| format!("{}", palette_stderr.warn.paint(b))),
+            );
+            if let Some(user) = state.repo.user() {
+                foreign_stacks.extend(
+                    git_stack::graph::trim_foreign_branches(
+                        &mut graph,
+                        &user,
+                        &[state.head_commit.id],
+                    )
                     .into_iter()
                     .map(|b| format!("{}", palette_stderr.warn.paint(b))),
-            );
+                );
+            }
         }
 
         if state.dry_run {
@@ -625,16 +656,28 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
 
     for graph in graphs {
         match state.show_format {
-            git_stack::config::Format::Silent => (),
-            git_stack::config::Format::Branches
-            | git_stack::config::Format::BranchCommits
-            | git_stack::config::Format::Commits => {
+            git_stack::config::Format::Silent => {}
+            git_stack::config::Format::List => {
+                let palette = if colored_stdout {
+                    Palette::colored()
+                } else {
+                    Palette::plain()
+                };
+                list(
+                    &mut std::io::stdout(),
+                    &state.repo,
+                    &graph,
+                    &state.protected_branches,
+                    &palette,
+                )?;
+            }
+            git_stack::config::Format::Graph => {
                 write!(
                     std::io::stdout(),
                     "{}",
                     DisplayTree::new(&state.repo, &graph)
                         .colored(colored_stdout)
-                        .show(state.show_format)
+                        .show(state.show_commits)
                         .stacked(state.show_stacked)
                         .protected_branches(&state.protected_branches)
                 )?;
@@ -722,8 +765,8 @@ fn git_prune_development(
     #[allow(clippy::needless_collect)]
     let remote_branches: Vec<_> = stdout
         .lines()
-        .filter_map(|l| l.rsplit_once('/'))
-        .map(|s| s.1)
+        .filter_map(|l| l.split_once('\t').map(|s| s.1))
+        .filter_map(|l| l.strip_prefix("refs/heads/"))
         .collect();
 
     for branch in branches {
@@ -794,20 +837,22 @@ fn git_push_node(
     let mut failed = Vec::new();
     for branch in node.branches.iter() {
         if node.pushable {
+            let raw_branch = repo
+                .raw()
+                .find_branch(&branch.name, git2::BranchType::Local)
+                .expect("all referenced branches exist");
+            let upstream_set = raw_branch.upstream().is_ok();
+
             let remote = repo.push_remote();
-            log::trace!(
-                "git push --force-with-lease --set-upstream {} {}",
-                remote,
-                branch.name
-            );
+            let mut args = vec!["push", "--force-with-lease"];
+            if !upstream_set {
+                args.push("--set-upstream");
+            }
+            args.push(remote);
+            args.push(&branch.name);
+            log::trace!("git {}", args.join(" "),);
             if !dry_run {
-                let status = std::process::Command::new("git")
-                    .arg("push")
-                    .arg("--force-with-lease")
-                    .arg("--set-upstream")
-                    .arg(repo.push_remote())
-                    .arg(&branch.name)
-                    .status();
+                let status = std::process::Command::new("git").args(&args).status();
                 match status {
                     Ok(status) => {
                         if !status.success() {
@@ -830,12 +875,41 @@ fn git_push_node(
     failed
 }
 
+fn list(
+    writer: &mut dyn std::io::Write,
+    repo: &git_stack::git::GitRepo,
+    graph: &git_stack::graph::Graph,
+    protected_branches: &git_stack::git::Branches,
+    palette: &Palette,
+) -> Result<(), std::io::Error> {
+    let head_branch = repo.head_branch().unwrap();
+    for node in graph.breadth_first_iter() {
+        let protected = protected_branches.get(node.commit.id);
+        let mut branches: Vec<_> = node.branches.iter().collect();
+        branches.sort();
+        for b in branches {
+            if protected.into_iter().flatten().contains(&b) {
+                // Base / protected branches are just show for context, they aren't part of the
+                // stack, so skip them here
+                continue;
+            }
+            writeln!(
+                writer,
+                "{}",
+                format_branch_name(b, node, &head_branch, protected_branches, palette)
+            )?;
+        }
+    }
+
+    Ok(())
+}
+
 struct DisplayTree<'r> {
     repo: &'r git_stack::git::GitRepo,
     graph: &'r git_stack::graph::Graph,
     protected_branches: git_stack::git::Branches,
     palette: Palette,
-    show: git_stack::config::Format,
+    show: git_stack::config::ShowCommits,
     stacked: bool,
 }
 
@@ -860,7 +934,7 @@ impl<'r> DisplayTree<'r> {
         self
     }
 
-    pub fn show(mut self, show: git_stack::config::Format) -> Self {
+    pub fn show(mut self, show: git_stack::config::ShowCommits) -> Self {
         self.show = show;
         self
     }
@@ -881,9 +955,8 @@ impl<'r> std::fmt::Display for DisplayTree<'r> {
         let head_branch = self.repo.head_branch().unwrap();
 
         let is_visible: Box<dyn Fn(&git_stack::graph::Node) -> bool> = match self.show {
-            git_stack::config::Format::Silent => unreachable!("No silent view for tree"),
-            git_stack::config::Format::Commits => Box::new(|_| true),
-            git_stack::config::Format::BranchCommits => Box::new(|node| {
+            git_stack::config::ShowCommits::All => Box::new(|_| true),
+            git_stack::config::ShowCommits::Unprotected => Box::new(|node| {
                 let interesting_commit = node.commit.id == head_branch.id
                     || node.commit.id == self.graph.root_id()
                     || node.children.is_empty();
@@ -891,14 +964,13 @@ impl<'r> std::fmt::Display for DisplayTree<'r> {
                 let protected = node.action.is_protected();
                 interesting_commit || !boring_commit || !protected
             }),
-            git_stack::config::Format::Branches => Box::new(|node| {
+            git_stack::config::ShowCommits::None => Box::new(|node| {
                 let interesting_commit = node.commit.id == head_branch.id
                     || node.commit.id == self.graph.root_id()
                     || node.children.is_empty();
                 let boring_commit = node.branches.is_empty() && node.children.len() == 1;
                 interesting_commit || !boring_commit
             }),
-            git_stack::config::Format::Debug => unreachable!("No debug view for tree"),
         };
 
         let mut tree = node_to_tree(
@@ -1299,7 +1371,7 @@ fn format_branch_status<'d>(
     // See format_commit_status
     if node.action.is_protected() {
         match commit_relation(repo, branch.id, branch.pull_id) {
-            Some((0, 0)) => format!(""),
+            Some((0, 0)) => String::new(),
             Some((local, 0)) => {
                 format!(" {}", palette.warn.paint(format!("({} ahead)", local)))
             }
@@ -1319,17 +1391,17 @@ fn format_branch_status<'d>(
             }
         }
     } else if node.action.is_delete() {
-        format!("")
+        String::new()
     } else if 1 < repo
         .raw()
         .find_commit(node.commit.id)
         .unwrap()
         .parent_count()
     {
-        format!("")
+        String::new()
     } else {
         if node.branches.is_empty() {
-            format!("")
+            String::new()
         } else {
             let branch = &node.branches[0];
             match commit_relation(repo, branch.id, branch.push_id) {
@@ -1354,7 +1426,7 @@ fn format_branch_status<'d>(
                     if node.pushable {
                         format!(" {}", palette.info.paint("(ready)"))
                     } else {
-                        format!("")
+                        String::new()
                     }
                 }
             }
@@ -1369,7 +1441,7 @@ fn format_commit_status<'d>(
 ) -> String {
     // See format_branch_status
     if node.action.is_protected() {
-        format!("")
+        String::new()
     } else if node.action.is_delete() {
         format!(" {}", palette.error.paint("(drop)"))
     } else if 1 < repo
@@ -1380,7 +1452,7 @@ fn format_commit_status<'d>(
     {
         format!(" {}", palette.error.paint("(merge commit)"))
     } else {
-        format!("")
+        String::new()
     }
 }
 
