@@ -120,7 +120,11 @@ fn mark_branch_protected(graph: &mut Graph, node_id: git2::Oid, branches: &mut V
     }
 }
 
-pub fn protect_old_branches(graph: &mut Graph, earlier_than: std::time::SystemTime) -> Vec<String> {
+pub fn protect_old_branches(
+    graph: &mut Graph,
+    earlier_than: std::time::SystemTime,
+    ignore: &[git2::Oid],
+) -> Vec<String> {
     let mut old_branches = Vec::new();
 
     let mut protected_queue = VecDeque::new();
@@ -139,7 +143,7 @@ pub fn protect_old_branches(graph: &mut Graph, earlier_than: std::time::SystemTi
             if child_action.is_protected() {
                 protected_queue.push_back(child_id);
             } else {
-                if is_branch_old(graph, child_id, earlier_than, &[]) {
+                if is_branch_old(graph, child_id, earlier_than, ignore) {
                     mark_branch_protected(graph, child_id, &mut old_branches);
                 }
             }
@@ -214,7 +218,11 @@ fn is_branch_old(
     true
 }
 
-pub fn protect_foreign_branches(graph: &mut Graph, user: &str) -> Vec<String> {
+pub fn protect_foreign_branches(
+    graph: &mut Graph,
+    user: &str,
+    ignore: &[git2::Oid],
+) -> Vec<String> {
     let mut foreign_branches = Vec::new();
 
     let mut protected_queue = VecDeque::new();
@@ -233,7 +241,7 @@ pub fn protect_foreign_branches(graph: &mut Graph, user: &str) -> Vec<String> {
             if child_action.is_protected() {
                 protected_queue.push_back(child_id);
             } else {
-                if !is_personal_branch(graph, child_id, user, &[]) {
+                if !is_personal_branch(graph, child_id, user, ignore) {
                     mark_branch_protected(graph, child_id, &mut foreign_branches);
                 }
             }
@@ -333,8 +341,8 @@ pub fn rebase_development_branches(graph: &mut Graph, new_base_id: git2::Oid) {
 
         if !rebaseable.is_empty() {
             let current = graph.get_mut(current_id).expect("all children exist");
-            for child_id in rebaseable.iter().copied() {
-                current.children.remove(&child_id);
+            for child_id in rebaseable.iter() {
+                current.children.remove(child_id);
             }
             graph
                 .get_mut(new_base_id)
@@ -380,8 +388,16 @@ pub fn pushable(graph: &mut Graph) {
     }
     while let Some((current_id, mut cause)) = node_queue.pop_front() {
         let current = graph.get_mut(current_id).expect("all children exist");
-        if !current.action.is_protected() {
-            if !current.branches.is_empty()
+        if current.action.is_protected() {
+            if !current.branches.is_empty() {
+                let branch = &current.branches[0];
+                log::debug!("{} isn't pushable, branch is protected", branch.name);
+                // Don't set `cause` as that will block descendants
+            }
+        } else {
+            if cause.is_some() {
+                // Preserve existing cause
+            } else if !current.branches.is_empty()
                 && current.branches.iter().all(|b| Some(b.id) == b.push_id)
             {
                 cause = Some("already pushed");
@@ -391,14 +407,14 @@ pub fn pushable(graph: &mut Graph) {
 
             if !current.branches.is_empty() {
                 let branch = &current.branches[0];
-                if let Some(cause) = cause {
-                    log::debug!("{} isn't pushable, {}", branch.name, cause);
+                if let Some(c) = cause {
+                    log::debug!("{} isn't pushable, {}", branch.name, c);
+                    cause = Some("parent isn't pushable");
                 } else {
                     log::debug!("{} is pushable", branch.name);
                     current.pushable = true;
+                    cause = Some("parent is pushable");
                 }
-                // Bail out, only the first branch of a stack is up for consideration
-                continue;
             }
         }
 
